@@ -19,7 +19,6 @@
 #include <gio/gio.h>      // for GApplication, G_APPLICATION
 #include <glib-object.h>  // for G_CALLBACK, g_signal_con...
 #include <glib.h>         // for GOptionEntry, gchar, G_O...
-#include <gtk/gtk.h>      // for gtk_dialog_add_button
 #include <libintl.h>      // for bindtextdomain, textdomain
 
 #include "control/RecentManager.h"           // for RecentManager
@@ -29,6 +28,7 @@
 #include "control/settings/Settings.h"       // for Settings
 #include "control/settings/SettingsEnums.h"  // for ICON_THEME_COLOR, ICON_T...
 #include "control/xojfile/LoadHandler.h"     // for LoadHandler
+#include "control/xojfile/SaveHandler.h"     // for SaveHandler
 #include "gui/GladeSearchpath.h"             // for GladeSearchpath
 #include "gui/MainWindow.h"                  // for MainWindow
 #include "gui/XournalView.h"                 // for XournalView
@@ -37,8 +37,8 @@
 #include "undo/UndoRedoHandler.h"            // for UndoRedoHandler
 #include "util/PathUtil.h"                   // for getConfigFolder, openFil...
 #include "util/PlaceholderString.h"          // for PlaceholderString
-#include "util/Stacktrace.h"                 // for Stacktrace
 #include "util/Util.h"                       // for execInUiThread
+#include "util/VersionInfo.h"                // for getVersionInfo
 #include "util/XojMsgBox.h"                  // for XojMsgBox
 #include "util/i18n.h"                       // for _, FS, _F
 
@@ -68,9 +68,6 @@ struct MigrateResult {
 
 auto migrateSettings() -> MigrateResult;
 
-void checkForErrorlog();
-void checkForEmergencySave(Control* control);
-
 void initResourcePath(GladeSearchpath* gladePath, const gchar* relativePathAndFile, bool failIfNotFound = true);
 
 void initCAndCoutLocales() {
@@ -88,9 +85,9 @@ auto migrateSettings() -> MigrateResult {
 
     if (!fs::exists(newConfigPath)) {
         const std::array oldPaths = {
-                Util::getConfigFolder().parent_path() /= "com.github.xournalpp.xournalpp",
-                Util::getConfigFolder().parent_path() /= "com.github.xournalpp.xournalpp.exe",
-                fs::u8path(g_get_home_dir()) /= ".xournalpp",
+                Util::getConfigFolder().parent_path() / "com.github.xournalpp.xournalpp",
+                Util::getConfigFolder().parent_path() / "com.github.xournalpp.xournalpp.exe",
+                Util::GFilename(g_get_home_dir()).toPath().value_or(fs::path()) / ".xournalpp",
         };
         for (auto const& oldPath: oldPaths) {
             if (!fs::is_directory(oldPath)) {
@@ -116,80 +113,14 @@ auto migrateSettings() -> MigrateResult {
     return {MigrateStatus::NotNeeded, ""};
 }
 
-static void deleteFile(const fs::path& file) {
+static void deleteFile(const fs::path& file, GtkWindow* win) {
     std::error_code error;
     if (!fs::remove(file, error)) {
         std::stringstream msg;
         msg << FS(_F("Failed to delete file: {1}") % file.u8string()) << std::endl;
         msg << error << std::endl << error.message() << std::endl;
         msg << FS(_F("Please delete the file manually"));
-        XojMsgBox::showErrorToUser(nullptr, msg.str());
-    }
-}
-
-void checkForErrorlog() {
-    std::vector<fs::path> errorList;
-
-    try {
-        const fs::path errorDir = Util::getCacheSubfolder(ERRORLOG_DIR);
-        if (!fs::exists(errorDir)) {
-            return;
-        }
-
-        // Todo(cpp20): replace std::chrono::hours(168) with std::chrono::weeks(1)
-        const auto oldestModificationDate = fs::file_time_type::clock::now() - std::chrono::hours(168);
-        for (auto const& f: fs::directory_iterator(errorDir)) {
-            if (f.is_regular_file() && f.path().filename().string().substr(0, 8) == "errorlog") {
-                if (f.last_write_time() > oldestModificationDate) {
-                    errorList.emplace_back(f);
-                }
-            }
-        }
-    } catch (fs::filesystem_error& e) {
-        g_warning("Filesystem error while looking for crash logs:\n"
-                  "   %s\n"
-                  "   %s\n",
-                  e.path1().c_str(), e.what());
-        return;
-    }
-
-    if (errorList.empty()) {
-        return;
-    }
-
-    std::sort(errorList.begin(), errorList.end());
-    std::string msg = errorList.size() == 1 ? _("There is a recent errorlogfile from Xournal++. Please file a "
-                                                "Bugreport, so the bug may be fixed.") :
-                                              _("There are recent errorlogfiles from Xournal++. Please file a "
-                                                "Bugreport, so the bug may be fixed.");
-    msg += "\n";
-    msg += FS(_F("The most recent log file name: {1}") % errorList[0].string());
-    msg += "\n\n";
-    msg += FS(_F("To prevent this popup from reappearing, simply delete the errorlogfiles."));
-
-    GtkWidget* dialog = gtk_message_dialog_new(nullptr, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s",
-                                               msg.c_str());
-
-    enum Responses { FILE_REPORT = 1, OPEN_FILE, OPEN_DIR, DELETE_FILE, CANCEL };
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Send Bugreport"), FILE_REPORT);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Open Logfile"), OPEN_FILE);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Open Logfile directory"), OPEN_DIR);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Delete Logfile"), DELETE_FILE);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Cancel"), CANCEL);
-
-    const int response = gtk_dialog_run(GTK_DIALOG(dialog));
-    gtk_widget_destroy(dialog);
-
-    auto const& errorlogPath = fs::path(errorList.front());
-    if (response == FILE_REPORT) {
-        Util::openFileWithDefaultApplication(PROJECT_BUGREPORT);
-        Util::openFileWithDefaultApplication(errorlogPath);
-    } else if (response == OPEN_FILE) {
-        Util::openFileWithDefaultApplication(errorlogPath);
-    } else if (response == OPEN_DIR) {
-        Util::openFileWithDefaultApplication(errorlogPath.parent_path());
-    } else if (response == DELETE_FILE) {
-        deleteFile(errorlogPath);
+        XojMsgBox::showErrorToUser(win, msg.str());
     }
 }
 
@@ -201,31 +132,23 @@ void checkForEmergencySave(Control* control) {
     }
 
     const std::string msg = _("Xournal++ crashed last time. Would you like to restore the last edited file?");
+    enum { DELETE_FILE = 1, RESTORE_FILE };
+    XojMsgBox::askQuestion(
+            control->getGtkWindow(), _("Recovery file detected"), msg,
+            {{_("Delete file"), DELETE_FILE}, {_("Restore file"), RESTORE_FILE}},
+            [file = std::move(file), ctrl = control](int response) mutable {
+                if (response == DELETE_FILE) {
+                    deleteFile(file, ctrl->getGtkWindow());
+                } else if (response == RESTORE_FILE) {
+                    ctrl->openFileWithoutSavingTheCurrentDocument(file, false, -1, [ctrl, file](bool) {
+                        ctrl->getDocument()->setFilepath("");
 
-    GtkWidget* dialog = gtk_message_dialog_new(nullptr, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "%s",
-                                               msg.c_str());
-
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Delete file"), 1);
-    gtk_dialog_add_button(GTK_DIALOG(dialog), _("Restore file"), 2);
-
-    const int res = gtk_dialog_run(GTK_DIALOG(dialog));
-
-    if (res == 1)  // Delete file
-    {
-        fs::remove(file);
-    } else if (res == 2)  // Open File
-    {
-        if (control->openFile(file, -1, true)) {
-            control->getDocument()->setFilepath("");
-
-            // Make sure the document is changed, there is a question to ask for save
-            control->getUndoRedoHandler()->addUndoAction(std::make_unique<EmergencySaveRestore>());
-            control->updateWindowTitle();
-            fs::remove(file);
-        }
-    }
-
-    gtk_widget_destroy(dialog);
+                        // Todo Make sure the document is changed + ask for saving
+                        ctrl->getUndoRedoHandler()->addUndoAction(std::make_unique<EmergencySaveRestore>());
+                        deleteFile(file, ctrl->getGtkWindow());
+                    });
+                }
+            });
 }
 
 namespace {
@@ -243,8 +166,8 @@ void exitOnMissingPdfFileName(const LoadHandler& loader) {
 
 /**
  * @brief Export the input file as a bunch of image files (one per page)
- * @param input Path to the input file
- * @param output Path to the output file(s)
+ * @param infile Path to the input file
+ * @param outfile Path to the output file(s)
  * @param range Page range to be parsed. If range=nullptr, exports the whole file
  * @param layerRange Layer range to be parsed. Will only export those layers, for every exported page.
  *                  If a number is too high for the number of layers on a given page, it is just ignored.
@@ -256,25 +179,64 @@ void exitOnMissingPdfFileName(const LoadHandler& loader) {
  *
  *  The priority is: pngDpi overwrites pngWidth overwrites pngHeight
  *
- * @return 0 on success, -2 on failure opening the input file, -3 on export failure
+ * @return 0 on success
+ *
+ * Calls std::exit(-2) on failure opening the input file and std::exit(-3) on export failure
  */
-auto exportImg(const char* input, const char* output, const char* range, const char* layerRange, int pngDpi,
-               int pngWidth, int pngHeight, ExportBackgroundType exportBackground) -> int {
+auto exportImg(fs::path infile, fs::path outfile, const char* range, const char* layerRange, int pngDpi, int pngWidth,
+               int pngHeight, ExportBackgroundType exportBackground) -> int {
+
     LoadHandler loader;
-    Document* doc = loader.loadDocument(input);
+
+    auto doc = loader.loadDocument(infile);
     if (doc == nullptr) {
-        g_error("%s", loader.getLastError().c_str());
+        g_message("%s", loader.getLastError().c_str());
+        std::exit(-2);
     }
 
     exitOnMissingPdfFileName(loader);
 
-    return ExportHelper::exportImg(doc, output, range, layerRange, pngDpi, pngWidth, pngHeight, exportBackground);
+    const auto res = ExportHelper::exportImg(doc.get(), std::move(outfile), range, layerRange, pngDpi, pngWidth,
+                                             pngHeight, exportBackground);
+    if (res != 0) {
+        std::exit(res);
+    }
+    return 0;
+}
+
+/**
+ * @brief Save a xopp-file with given pdf-background
+ *
+ * @param infile Path to the input .pdf file
+ * @param outfile Path to the output .xopp file
+ * @return int 0 on success
+ *
+ * Calls std::exit(-2) on failure reading the input file and std::exit(-3) on save failure
+ */
+auto saveDoc(fs::path infile, fs::path outfile) -> int {
+    SaveHandler saver;
+    auto handler = std::make_unique<DocumentHandler>();
+    auto newDoc = std::make_unique<Document>(handler.get());
+    const bool res = newDoc->readPdf(infile, /*initPages=*/true, false);
+    if (!res) {
+        g_message("%s", FC(_F("Error: {1}") % newDoc->getLastErrorMsg().c_str()));
+        std::exit(-2);
+    }
+    const fs::path out = fs::absolute(outfile);
+    saver.prepareSave(newDoc.get(), out);
+    saver.saveTo(out);
+
+    if (!saver.getErrorMessage().empty()) {
+        g_message("%s", FC(_F("Error: {1}") % saver.getErrorMessage()));
+        std::exit(-3);
+    }
+    return 0;
 }
 
 /**
  * @brief Export the input file as pdf
- * @param input Path to the input file
- * @param output Path to the output file
+ * @param infile Path to the input file
+ * @param outfile Path to the output file
  * @param layerRange Layer range to be parsed. Will only export those layers, for every exported page.
  *                  If a number is too high for the number of layers on a given page, it is just ignored.
  *                  If range=nullptr, exports all layers.
@@ -282,20 +244,29 @@ auto exportImg(const char* input, const char* output, const char* range, const c
  * @param exportBackground If EXPORT_BACKGROUND_NONE, the exported pdf file has white background
  * @param progressiveMode If true, then for each xournalpp page, instead of rendering one PDF page, the page layers are
  * rendered one by one to produce as many pages as there are layers.
+ * @param backend The requested backend
  *
- * @return 0 on success, -2 on failure opening the input file, -3 on export failure
+ * @return 0 on success
+ *
+ * Calls std::exit(-2) on failure opening the input file and std::exit(-3) on export failure
  */
-auto exportPdf(const char* input, const char* output, const char* range, const char* layerRange,
-               ExportBackgroundType exportBackground, bool progressiveMode) -> int {
+auto exportPdf(fs::path infile, fs::path outfile, const char* range, const char* layerRange,
+               ExportBackgroundType exportBackground, bool progressiveMode, ExportBackend backend) -> int {
     LoadHandler loader;
-    Document* doc = loader.loadDocument(input);
+    auto doc = loader.loadDocument(infile);
     if (doc == nullptr) {
-        g_error("%s", loader.getLastError().c_str());
+        g_message("%s", loader.getLastError().c_str());
+        std::exit(-2);
     }
 
     exitOnMissingPdfFileName(loader);
 
-    return ExportHelper::exportPdf(doc, output, range, layerRange, exportBackground, progressiveMode);
+    const auto res =
+            ExportHelper::exportPdf(doc.get(), outfile, range, layerRange, exportBackground, progressiveMode, backend);
+    if (res != 0) {
+        std::exit(res);
+    }
+    return 0;
 }
 
 struct XournalMainPrivate {
@@ -309,11 +280,13 @@ struct XournalMainPrivate {
         g_strfreev(optFilename);
         g_free(pdfFilename);
         g_free(imgFilename);
+        g_free(docFilename);
     }
 
-    gchar** optFilename{};
-    gchar* pdfFilename{};
-    gchar* imgFilename{};
+    gchar** optFilename{};  ///< Array of paths, in GFilename encoding
+    gchar* pdfFilename{};   ///< Single path, in GFilename encoding
+    gchar* imgFilename{};   ///< Single path, in GFilename encoding
+    gchar* docFilename{};   ///< Single path, in GFilename encoding
     gboolean showVersion = false;
     int openAtPageNumber = 0;  // when no --page is used, the document opens at the page specified in the metadata file
     gchar* exportRange{};
@@ -325,6 +298,8 @@ struct XournalMainPrivate {
     gboolean exportNoRuling = false;
     gboolean progressiveMode = false;
     gboolean disableAudio = false;
+    gboolean attachMode = false;
+    gchar* exportPdfBackend{};
     std::unique_ptr<GladeSearchpath> gladePath;
     std::unique_ptr<Control> control;
     std::unique_ptr<MainWindow> win;
@@ -368,7 +343,7 @@ auto findResourcePath(const fs::path& searchFile) -> fs::path {
             return *path;
         }*/
     /// real execution path
-    if (auto path = search_for(Stacktrace::getExePath().parent_path()); path) {
+    if (auto path = search_for(Util::getExePath().parent_path()); path) {
         return *path;
     }
     // Not found
@@ -396,15 +371,13 @@ void initResourcePath(GladeSearchpath* gladePath, const gchar* relativePathAndFi
     std::string msg =
             FS(_F("<span foreground='red' size='x-large'>Missing the needed UI file:\n<b>{1}</b></span>\nCould "
                   "not find them at any location.\n  Not relative\n  Not in the Working Path\n  Not in {2}") %
-               relativePathAndFile % Util::getDataPath().string());
+               relativePathAndFile % Util::getDataPath().u8string());
 
     if (!failIfNotFound) {
         msg += _("\n\nWill now attempt to run without this file.");
-    }
-    XojMsgBox::showErrorToUser(nullptr, msg);
-
-    if (failIfNotFound) {
-        exit(12);
+        XojMsgBox::showErrorToUser(nullptr, msg);
+    } else {
+        XojMsgBox::showErrorAndQuit(msg, 12);
     }
 }
 
@@ -428,11 +401,11 @@ void on_open_files(GApplication* application, gpointer f, gint numFiles, gchar*
         XojMsgBox::showErrorToUser(GTK_WINDOW(app_data->win->getWindow()), msg);
     }
 
-    const fs::path p = Util::fromGFilename(g_file_get_path(files[0]), false);
+    const fs::path p = Util::fromGFile(files[0]);
 
     try {
         if (fs::exists(p)) {
-            app_data->control->openFile(p);
+            app_data->control->openFile(fs::absolute(p));
         } else {
             const std::string msg = FS(_F("File {1} does not exist.") % p.u8string());
             XojMsgBox::showErrorToUser(GTK_WINDOW(app_data->win->getWindow()), msg);
@@ -455,6 +428,7 @@ void on_startup(GApplication* application, XMPtr app_data) {
     app_data->gladePath = std::make_unique<GladeSearchpath>();
     initResourcePath(app_data->gladePath.get(), "ui/about.glade");
     initResourcePath(app_data->gladePath.get(), "ui/xournalpp.css", false);
+    initResourcePath(app_data->gladePath.get(), "ui/toolbar.ini", false);
 
     app_data->control = std::make_unique<Control>(application, app_data->gladePath.get(), app_data->disableAudio);
 
@@ -468,78 +442,68 @@ void on_startup(GApplication* application, XMPtr app_data) {
     app_data->win = std::make_unique<MainWindow>(app_data->gladePath.get(), app_data->control.get(),
                                                  GTK_APPLICATION(application));
     app_data->control->initWindow(app_data->win.get());
+    app_data->win->populate(app_data->gladePath.get());
 
     if (migrateResult.status != MigrateStatus::NotNeeded) {
         Util::execInUiThread(
                 [=]() { XojMsgBox::showErrorToUser(app_data->control->getGtkWindow(), migrateResult.message); });
     }
 
+    gtk_application_set_menubar(GTK_APPLICATION(application), app_data->win->getMenuModel());
+    // Do we want stuff in gtk_application_set_app_menu?
+
     app_data->win->show(nullptr);
 
-    bool opened = false;
+    fs::path p;
     if (app_data->optFilename) {
         if (g_strv_length(app_data->optFilename) != 1) {
             const std::string msg = _("Sorry, Xournal++ can only open one file at once.\n"
                                       "Others are ignored.");
             XojMsgBox::showErrorToUser(GTK_WINDOW(app_data->win->getWindow()), msg);
         }
-
-        fs::path p = Util::fromGFilename(app_data->optFilename[0], false);
-
-
+        p = Util::fromGFilename(app_data->optFilename[0]);
         try {
             p = fs::absolute(p);
-            if (fs::exists(p)) {
-                opened = app_data->control->openFile(p,
-                                                     app_data->openAtPageNumber - 1);  // First page for user is page 1
-            } else {
-                opened = app_data->control->newFile("", p);
-            }
         } catch (const fs::filesystem_error& e) {
-            const std::string msg = FS(_F("Filesystem error: {1}\n"
-                                          "Sorry, Xournal++ cannot open the file: {2}\n"
-                                          "Consider copying the file to a local directory.") %
-                                       e.what() % p.u8string());
-            XojMsgBox::showErrorToUser(GTK_WINDOW(app_data->win->getWindow()), msg);
-            opened = app_data->control->newFile("", p);
+            g_warning("Unable to convert path %s to absolute path: %s", app_data->optFilename[0], e.what());
         }
     } else if (app_data->control->getSettings()->isAutoloadMostRecent()) {
         auto most_recent = RecentManager::getMostRecent();
         if (most_recent) {
-            if (auto p = Util::fromUri(gtk_recent_info_get_uri(most_recent.get()))) {
-                opened = app_data->control->openFile(*p);
+            if (auto opt = Util::fromUri(gtk_recent_info_get_uri(most_recent.get()))) {
+                p = opt.value();
+            }
+            if (std::error_code err; !fs::exists(p, err)) {
+                if (err) {
+                    g_warning("Failed to determine if recent path exists \"%s\": %s", char_cast(p.u8string().c_str()),
+                              err.message().c_str());
+                } else {
+                    g_warning("Tried to open the most recent file but it no longer exists:\n\"%s\"",
+                              char_cast(p.u8string().c_str()));
+                }
+                p = fs::path();
             }
         }
     }
 
-    app_data->control->getScheduler()->start();
+    app_data->control->openFileWithoutSavingTheCurrentDocument(
+            std::move(p), app_data->attachMode, app_data->openAtPageNumber - 1,
+            [ctrl = app_data->control.get(), app = GTK_APPLICATION(application)](bool) {
+                ctrl->getScheduler()->start();
 
-    if (!opened) {
-        app_data->control->newFile();
-    }
+                checkForEmergencySave(ctrl);
 
-    checkForErrorlog();
-    checkForEmergencySave(app_data->control.get());
-
-    // There is a timing issue with the layout
-    // This fixes it, see #405
-    Util::execInUiThread([=]() { app_data->control->getWindow()->getXournal()->layoutPages(); });
-    gtk_application_add_window(GTK_APPLICATION(application), GTK_WINDOW(app_data->win->getWindow()));
+                // There is a timing issue with the layout
+                // This fixes it, see #405
+                Util::execInUiThread([ctrl]() { ctrl->getWindow()->getXournal()->layoutPages(); });
+                gtk_application_add_window(app, ctrl->getGtkWindow());
+            });
 }
 
 auto on_handle_local_options(GApplication*, GVariantDict*, XMPtr app_data) -> gint {
     initCAndCoutLocales();
 
-    auto print_version = [&] {
-        if (!std::string(GIT_COMMIT_ID).empty()) {
-            std::cout << PROJECT_NAME << " " << PROJECT_VERSION << " (" << GIT_COMMIT_ID << ")" << std::endl;
-        } else {
-            std::cout << PROJECT_NAME << " " << PROJECT_VERSION << std::endl;
-        }
-        std::cout << "└──libgtk: " << gtk_get_major_version() << "."  //
-                  << gtk_get_minor_version() << "."                   //
-                  << gtk_get_micro_version() << std::endl;            //
-    };
+    auto print_version = [&] { std::cout << xoj::util::getVersionInfo() << std::endl; };
 
     auto exec_guarded = [&](auto&& fun, auto&& s) {
         try {
@@ -565,19 +529,21 @@ auto on_handle_local_options(GApplication*, GVariantDict*, XMPtr app_data) -> gi
     if (app_data->pdfFilename && app_data->optFilename && *app_data->optFilename) {
         return exec_guarded(
                 [&] {
-                    return exportPdf(*app_data->optFilename, app_data->pdfFilename, app_data->exportRange,
+                    return exportPdf(Util::fromGFilename(*app_data->optFilename),
+                                     Util::fromGFilename(app_data->pdfFilename), app_data->exportRange,
                                      app_data->exportLayerRange,
                                      app_data->exportNoBackground ? EXPORT_BACKGROUND_NONE :
                                      app_data->exportNoRuling     ? EXPORT_BACKGROUND_UNRULED :
                                                                     EXPORT_BACKGROUND_ALL,
-                                     app_data->progressiveMode);
+                                     app_data->progressiveMode, ExportBackend::fromString(app_data->exportPdfBackend));
                 },
                 "exportPdf");
     }
     if (app_data->imgFilename && app_data->optFilename && *app_data->optFilename) {
         return exec_guarded(
                 [&] {
-                    return exportImg(*app_data->optFilename, app_data->imgFilename, app_data->exportRange,
+                    return exportImg(Util::fromGFilename(*app_data->optFilename),
+                                     Util::fromGFilename(app_data->imgFilename), app_data->exportRange,
                                      app_data->exportLayerRange, app_data->exportPngDpi, app_data->exportPngWidth,
                                      app_data->exportPngHeight,
                                      app_data->exportNoBackground ? EXPORT_BACKGROUND_NONE :
@@ -586,6 +552,14 @@ auto on_handle_local_options(GApplication*, GVariantDict*, XMPtr app_data) -> gi
                 },
                 "exportImg");
     }
+    if (app_data->docFilename && app_data->optFilename && *app_data->optFilename) {
+        return exec_guarded(
+                [&] {
+                    return saveDoc(Util::fromGFilename(*app_data->optFilename),
+                                   Util::fromGFilename(app_data->docFilename));
+                },
+                "saveDocument");
+    }
     return -1;
 }
 
@@ -605,7 +579,7 @@ void XournalMain::initLocalisation() {
 #ifdef _WIN32
     wbindtextdomain(GETTEXT_PACKAGE, localeDir.wstring().c_str());
 #else
-    bindtextdomain(GETTEXT_PACKAGE, localeDir.u8string().c_str());
+    bindtextdomain(GETTEXT_PACKAGE, char_cast(localeDir.u8string().c_str()));
 #endif
 
     textdomain(GETTEXT_PACKAGE);
@@ -630,7 +604,7 @@ auto XournalMain::run(int argc, char** argv) -> int {
 
     XournalMainPrivate app_data;
     GtkApplication* app = gtk_application_new("com.github.xournalpp.xournalpp", APP_FLAGS);
-    g_object_set(G_OBJECT(app), "register-session", true, NULL);  // Needed for opening files on MacOS from Finder
+    g_object_set(G_OBJECT(app), "register-session", true, nullptr);  // Needed for opening files on MacOS from Finder
     g_set_prgname("com.github.xournalpp.xournalpp");
     g_signal_connect(app, "activate", G_CALLBACK(&on_activate), &app_data);
     g_signal_connect(app, "command-line", G_CALLBACK(&on_command_line), &app_data);
@@ -647,59 +621,78 @@ auto XournalMain::run(int argc, char** argv) -> int {
                                        _("Get version of xournalpp"), nullptr},
                           GOptionEntry{"disable-audio", 0, 0, G_OPTION_ARG_NONE, &app_data.disableAudio,
                                        _("Disable audio for this session"), nullptr},
+                          GOptionEntry{"attach-mode", 0, 0, G_OPTION_ARG_NONE, &app_data.attachMode,
+                                       _("Open PDF in attach mode\n"
+                                         "                                       Ignored if no PDF file is specified."),
+                                       nullptr},
+                          GOptionEntry{"save", 's', 0, G_OPTION_ARG_FILENAME, &app_data.docFilename,
+                                       _("Save xopp-file with the background PDF specified as FILE"), "XOPPFILE"},
                           GOptionEntry{nullptr}};  // Must be terminated by a nullptr. See gtk doc
     g_application_add_main_option_entries(G_APPLICATION(app), options.data());
 
+    std::string pdfbackendMessage =
+            FS(_F("Use the provided backend for PDF exports.\n"
+                  "                                       Available backends: {1}\n"
+                  "                                       No effect without -p/--create-pdf=foo.pdf") %
+               ExportBackend::listAvailableBackends());
     /**
      * Export related options
      */
     std::array exportOptions = {
             GOptionEntry{"create-pdf", 'p', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_FILENAME, &app_data.pdfFilename,
                          _("Export FILE as PDF"), "PDFFILE"},
-            GOptionEntry{"create-img", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_FILENAME, &app_data.imgFilename,
-                         _("Export FILE as image files (one per page)\n"
-                           "                                 Guess the output format from the extension of IMGFILE\n"
-                           "                                 Supported formats: .png, .svg"),
-                         "IMGFILE"},
-            GOptionEntry{"export-no-background", 0, 0, G_OPTION_ARG_NONE, &app_data.exportNoBackground,
-                         _("Export without background\n"
-                           "                                 The exported file has transparent or white background,\n"
-                           "                                 depending on what its format supports\n"),
-                         0},
+            GOptionEntry{
+                    "create-img", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_FILENAME, &app_data.imgFilename,
+                    _("Export FILE as image files (one per page)\n"
+                      "                                       Guess the output format from the extension of IMGFILE\n"
+                      "                                       Supported formats: .png, .svg"),
+                    "IMGFILE"},
+            GOptionEntry{
+                    "export-no-background", 0, 0, G_OPTION_ARG_NONE, &app_data.exportNoBackground,
+                    _("Export without background\n"
+                      "                                       The exported file has transparent or white background,\n"
+                      "                                       depending on what its format supports\n"),
+                    0},
             GOptionEntry{"export-no-ruling", 0, 0, G_OPTION_ARG_NONE, &app_data.exportNoRuling,
                          _("Export without ruling\n"
-                           "                                 The exported file has no paper ruling\n"),
+                           "                                       The exported file has no paper ruling\n"),
                          0},
-            GOptionEntry{"export-layers-progressively", 0, 0, G_OPTION_ARG_NONE, &app_data.progressiveMode,
-                         _("Export layers progressively\n"
-                           "                                 In PDF export, Render layers progressively one by one.\n"
-                           "                                 This results in N export pages per page with N layers,\n"
-                           "                                 building up the layer stack progressively.\n"
-                           "                                 The resulting PDF file can be used for a presentation.\n"),
-                         0},
-            GOptionEntry{"export-range", 0, 0, G_OPTION_ARG_STRING, &app_data.exportRange,
-                         _("Only export the pages specified by RANGE (e.g. \"2-3,5,7-\")\n"
-                           "                                 No effect without -p/--create-pdf or -i/--create-img"),
-                         "RANGE"},
-            GOptionEntry{"export-layer-range", 0, 0, G_OPTION_ARG_STRING, &app_data.exportLayerRange,
-                         _("Only export the layers specified by RANGE (e.g. \"2-3,5,7-\")\n"
-                           "                                 No effect without -p/--create-pdf or -i/--create-img"),
-                         "RANGE"},
+            GOptionEntry{
+                    "export-layers-progressively", 0, 0, G_OPTION_ARG_NONE, &app_data.progressiveMode,
+                    _("Export layers progressively\n"
+                      "                                       In PDF export, Render layers progressively one by one.\n"
+                      "                                       This results in N export pages per page with N layers,\n"
+                      "                                       building up the layer stack progressively.\n"
+                      "                                       The resulting PDF file can be used for a "
+                      "presentation.\n"),
+                    0},
+            GOptionEntry{
+                    "export-range", 0, 0, G_OPTION_ARG_STRING, &app_data.exportRange,
+                    _("Only export the pages specified by RANGE (e.g. \"2-3,5,7-\")\n"
+                      "                                       No effect without -p/--create-pdf or -i/--create-img"),
+                    "RANGE"},
+            GOptionEntry{
+                    "export-layer-range", 0, 0, G_OPTION_ARG_STRING, &app_data.exportLayerRange,
+                    _("Only export the layers specified by RANGE (e.g. \"2-3,5,7-\")\n"
+                      "                                       No effect without -p/--create-pdf or -i/--create-img"),
+                    "RANGE"},
             GOptionEntry{"export-png-dpi", 0, 0, G_OPTION_ARG_INT, &app_data.exportPngDpi,
                          _("Set DPI for PNG exports. Default is 300\n"
-                           "                                 No effect without -i/--create-img=foo.png"),
+                           "                                       No effect without -i/--create-img=foo.png"),
                          "N"},
             GOptionEntry{"export-png-width", 0, 0, G_OPTION_ARG_INT, &app_data.exportPngWidth,
                          _("Set page width for PNG exports\n"
-                           "                                 No effect without -i/--create-img=foo.png\n"
-                           "                                 Ignored if --export-png-dpi is used"),
+                           "                                       No effect without -i/--create-img=foo.png\n"
+                           "                                       Ignored if --export-png-dpi is used"),
                          "N"},
-            GOptionEntry{
-                    "export-png-height", 0, 0, G_OPTION_ARG_INT, &app_data.exportPngHeight,
-                    _("Set page height for PNG exports\n"
-                      "                                 No effect without -i/--create-img=foo.png\n"
-                      "                                 Ignored if --export-png-dpi or --export-png-width is used"),
-                    "N"},
+            GOptionEntry{"export-png-height", 0, 0, G_OPTION_ARG_INT, &app_data.exportPngHeight,
+                         _("Set page height for PNG exports\n"
+                           "                                       No effect without -i/--create-img=foo.png\n"
+                           "                                       Ignored if --export-png-dpi or --export-png-width "
+                           "is used"),
+                         "N"},
+            GOptionEntry{"export-pdf-backend", 0, 0, G_OPTION_ARG_STRING, &app_data.exportPdfBackend,
+                         pdfbackendMessage.c_str(), "BACKEND"},
             GOptionEntry{nullptr}};  // Must be terminated by a nullptr. See gtk doc
     GOptionGroup* exportGroup = g_option_group_new("export", _("Advanced export options"),
                                                    _("Display advanced export options"), nullptr, nullptr);
