@@ -13,6 +13,8 @@
 
 #include <memory>  // for unique_ptr
 
+#include "PdfExportBackend.h"
+
 class Document;
 class ProgressListener;
 class XojPdfExport;
@@ -20,10 +22,9 @@ class XojPdfExport;
 class XojPdfExportFactory {
 private:
     XojPdfExportFactory();
-    virtual ~XojPdfExportFactory();
+    ~XojPdfExportFactory();
 
 public:
-    static std::unique_ptr<XojPdfExport> createExport(Document* doc, ProgressListener* listener);
-
-private:
+    static std::unique_ptr<XojPdfExport> createExport(const Document* doc, ProgressListener* listener,
+                                                      ExportBackend backend = ExportBackend::DEFAULT);
 };
