@@ -1,4 +1,5 @@
 #include <array>
+#include <cstdint>
 #include <random>
 #include <string>
 #include <tuple>
@@ -8,6 +9,7 @@
 #include <gtest/gtest.h>
 
 #include "model/Stroke.h"
+#include "util/StringUtils.h"
 #include "util/serializing/BinObjectEncoding.h"
 #include "util/serializing/HexObjectEncoding.h"
 #include "util/serializing/ObjectInputStream.h"
@@ -18,7 +20,7 @@ template <typename T>
 std::string serializeDataVector(const std::vector<T>& data) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     outStream.writeData(data);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -28,7 +30,7 @@ std::string serializeImage(cairo_surface_t* surf) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     std::string data{reinterpret_cast<char*>(cairo_image_surface_get_data(surf))};
     outStream.writeImage(data);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -37,7 +39,7 @@ std::string serializeImage(cairo_surface_t* surf) {
 std::string serializeString(const std::string& str) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     outStream.writeString(str);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -46,7 +48,7 @@ std::string serializeString(const std::string& str) {
 std::string serializeSizeT(size_t x) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     outStream.writeSizeT(x);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -55,7 +57,7 @@ std::string serializeSizeT(size_t x) {
 std::string serializeDouble(double x) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     outStream.writeDouble(x);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -64,7 +66,16 @@ std::string serializeDouble(double x) {
 std::string serializeInt(int x) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     outStream.writeInt(x);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
+    auto resStr = std::string{outStr->str, outStr->len};
+    g_string_free(outStr, true);
+    return resStr;
+}
+
+std::string serializeUInt(uint32_t x) {
+    ObjectOutputStream outStream(new BinObjectEncoding);
+    outStream.writeUInt(x);
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -73,7 +84,7 @@ std::string serializeInt(int x) {
 std::string serializeStroke(Stroke& stroke) {
     ObjectOutputStream outStream(new BinObjectEncoding);
     stroke.serialize(outStream);
-    auto outStr = outStream.getStr();
+    auto outStr = outStream.stealData();
     auto resStr = std::string{outStr->str, outStr->len};
     g_string_free(outStr, true);
     return resStr;
@@ -83,7 +94,7 @@ template <typename T>
 void testReadDataType(const std::vector<T>& data) {
     std::string str = serializeDataVector<T>(data);
     ObjectInputStream stream;
-    EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+    EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
 
     std::vector<T> outputData;
     stream.readData(outputData);
@@ -93,7 +104,7 @@ void testReadDataType(const std::vector<T>& data) {
 
 
 TEST(UtilObjectIOStream, testReadData) {
-    testReadDataType(std::vector<char>{0, 42, -42});
+    testReadDataType(std::vector<char>{0, 0x0e, 0x3f});
     testReadDataType(std::vector<long>{0, 42, -42});
     testReadDataType(std::vector<long long>{0, 420000000000, -42000000000});
     testReadDataType(std::vector<float>{0, 42., -42.});
@@ -111,7 +122,7 @@ TEST(UtilObjectIOStream, testReadData) {
 TEST(UtilObjectIOStream, testReadImage) {
     // Generate a "random" image and serialize/deserialize it.
     std::mt19937 gen(4242);
-    std::uniform_int_distribution<unsigned char> distrib(0, 255);
+    std::uniform_int_distribution<uint16_t> distrib(0, 255);
 
     const cairo_format_t format = CAIRO_FORMAT_ARGB32;
     cairo_surface_t* surface = cairo_image_surface_create(format, 800, 800);
@@ -125,7 +136,7 @@ TEST(UtilObjectIOStream, testReadImage) {
     std::string strSurface = serializeImage(surface);
 
     ObjectInputStream stream;
-    EXPECT_TRUE(stream.read(&strSurface[0], (int)strSurface.size() + 1));
+    EXPECT_TRUE(stream.read(&strSurface[0], strSurface.size() + 1));
 
     std::string outputStr = stream.readImage();
 
@@ -149,7 +160,11 @@ TEST(UtilObjectIOStream, testReadImage) {
 
 TEST(UtilObjectIOStream, testReadString) {
     std::vector<std::string> stringToTest{
-            "", "Hello World", XML_VERSION_STR, "1337",
+            "",
+            "Hello World",
+            XML_VERSION_STR,
+            "1337",
+            char_cast(u8"Ṯℏiⓢ is UTF∞"),
             "Laborum beatae sit at. Tempore ex odio et non et iste et. Deleniti magni beatae quod praesentium dicta quas ducimus hic. Nemo vel est saepe voluptatibus. Sunt eveniet aut saepe consequatur fuga ad molestias.\n \
                 Culpa nulla saepe alias magni nemo magni. Sed sit sint repellat doloremque. Quo ipsum debitis quos impedit. Omnis expedita veritatis nihil sint et itaque possimus. Nobis est fugit vel omnis. Dolores architecto laudantium nihil rerum."};
 
@@ -163,7 +178,7 @@ TEST(UtilObjectIOStream, testReadString) {
 
         ObjectInputStream stream;
         // The +1 stands for the \0 character
-        EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+        EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
         std::string output = stream.readString();
         EXPECT_EQ(x, output);
     }
@@ -182,7 +197,7 @@ TEST(UtilObjectIOStream, testReadSizeT) {
 
         ObjectInputStream stream;
         // The +1 stands for the \0 character
-        EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+        EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
         size_t output = stream.readSizeT();
         EXPECT_EQ(x, output);
     }
@@ -201,12 +216,32 @@ TEST(UtilObjectIOStream, testReadInt) {
 
         ObjectInputStream stream;
         // The +1 stands for the \0 character
-        EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+        EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
         int output = stream.readInt();
         EXPECT_EQ(x, output);
     }
 }
 
+TEST(UtilObjectIOStream, testReadUInt) {
+    std::vector<uint32_t> uintToTest{0, 1, 42, 144, 65000, 12345678, 4294967295};
+
+    std::vector<std::pair<std::string, uint32_t>> testData;
+    testData.reserve(uintToTest.size());
+    for (auto&& number: uintToTest) {
+        testData.emplace_back(serializeUInt(number), number);
+    }
+
+    for (auto&& data: testData) {
+        std::string& str = data.first;
+        uint32_t x = data.second;
+
+        ObjectInputStream stream;
+        // The +1 stands for the \0 character
+        EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
+        uint32_t output = stream.readUInt();
+        EXPECT_EQ(x, output);
+    }
+}
 
 TEST(UtilObjectIOStream, testReadDouble) {
     std::vector<double> doubleToTest{0., 0.5, 42., 46.5, -85.2, -1337, 1e50};
@@ -221,7 +256,7 @@ TEST(UtilObjectIOStream, testReadDouble) {
 
         ObjectInputStream stream;
         // The +1 stands for the \0 character
-        EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+        EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
         double output = stream.readDouble();
         EXPECT_DOUBLE_EQ(dbl, output);
     }
@@ -254,12 +289,12 @@ TEST(UtilObjectIOStream, testReadComplexObject) {
             outStream.writeDouble(-d);
             outStream.endObject();
 
-            auto gstr = outStream.getStr();
+            auto gstr = outStream.stealData();
             std::string str(gstr->str, gstr->len);
             g_string_free(gstr, true);
 
             ObjectInputStream stream;
-            EXPECT_TRUE(stream.read(&str[0], (int)str.size() + 1));
+            EXPECT_TRUE(stream.read(&str[0], str.size() + 1));
 
             std::string outputName = stream.readObject();
             EXPECT_EQ(outputName, objectName);
@@ -317,7 +352,7 @@ void assertStrokeEquality(const Stroke& stroke1, const Stroke& stroke2) {
 }
 
 TEST(UtilObjectIOStream, testReadStroke) {
-    std::vector<Stroke> strokes(8);
+    std::vector<Stroke> strokes(9);
     // strokes[0]: empty stroke
 
     strokes[1].addPoint(Point(42, 42));
@@ -351,12 +386,15 @@ TEST(UtilObjectIOStream, testReadStroke) {
     strokes[7].setToolType(static_cast<StrokeTool::Value>(42));
     strokes[7].setWidth(-1337.);
 
+    // Test UTF8 paths
+    strokes[8].setAudioFilename(u8"🡶⬶/assets/foobar.mp3");
+
     size_t i = 0;
     try {
         for (auto&& stroke: strokes) {
             std::string out_string = serializeStroke(stroke);
             ObjectInputStream istream;
-            istream.read(out_string.c_str(), (int)out_string.size());
+            istream.read(out_string.c_str(), out_string.size());
 
             Stroke in_stroke;
             in_stroke.readSerialized(istream);
